/*
    tsync - a utility to sync remote encrypted copies of a tstore
    Copyright (C) 2005  Kasper Dupont

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    with permission from the author any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

    $Id: tsync-util.c,v 1.5 2005/06/15 04:48:57 kasperd Exp $
*/

#include "tsync.h"

int start_helper(char ** cmd)
{
  int socket_fd[2];
  if (socketpair(AF_UNIX,SOCK_STREAM,0,socket_fd)) {
    perror("socketpair");
    exit(EXIT_FAILURE);
  }
  switch(fork()) {
  case 0:
    close(socket_fd[0]);
    dup2(socket_fd[1],0);
    dup2(socket_fd[1],1);
    close(socket_fd[1]);
    execvp(cmd[0],cmd);
    perror(cmd[0]);
    _exit(EXIT_FAILURE);
  case -1:
    perror("fork");
    exit(EXIT_FAILURE);
  }
  close(socket_fd[1]);
  fcntl(socket_fd[1],F_SETFD,1); /* close-on-exec */
  return socket_fd[0];
}

int start_server(char ** cmd, uint32_t *record_count)
{
  uint32_t rc;
  int server_fd=start_helper(cmd);
  assert(write(server_fd,getenv("TSYNC_COOKIE"),43)==43);
  assert(read(server_fd,&rc,4)==4);
  if (record_count) *record_count=ntohl(rc);
  return server_fd;
}

int start_gpg(char * pass, int encrypt)
{
  int r,l=strlen(pass);
  int pass_pipe[2];
  char fdbuf[42];
  char *gpg_cmd[]={"gpg","--symmetric","--no-tty","--cipher-algo","AES256","--passphrase-fd",fdbuf,NULL};
  assert(encrypt);
  if (pipe(pass_pipe)) {
    perror("pipe");
    exit(EXIT_FAILURE);
  }
  fcntl(pass_pipe[1],F_SETFD,1); /* close-on-exec */
  sprintf(fdbuf,"%d",pass_pipe[0]);
  r=start_helper(gpg_cmd);
  close(pass_pipe[0]);
  assert(write(pass_pipe[1],pass,l)==l);
  close(pass_pipe[1]);
  return r;
}
